## Formula Viewer
## v 2.0.3
##
## 2012 - Jun - 19
## Added 4 sides.
## 
## Author: A. Roslovtsev
## Email: andrei[AT]bytes-and-pixels[DOT]com
## Web: www.bytes-and-pixels.com
## 
## This tool is a helper for Vector Path plug-in for Gimp
## It allows to preview your curve interactively
## 
## This script is free software under GPL license.

import sys, csv
from os import path
from Tkinter import *
import tkMessageBox as msg
from math    import *

class fview():

    def __init__(self,Width=400, Height=400):
            self.offset_points = 0
            #---------------------------------------------------
            # Load preset data if exists, else create empty file
            def open_presets():
                self.presets = []
                
                if path.isfile(sys.path[0]+'/formula_presets2.csv')!= True:
                    f = open(sys.path[0]+'/formula_presets2.csv','w')
                    f.close

                f =  open(sys.path[0]+'/formula_presets2.csv','r')
                data = csv.reader(f, delimiter=',',quotechar='"')
                for datum in data:
                    # if csv has insufficient number of fields,
                    # (comes from earlier version) - append zeros
                    if len(datum) < 15:
                        i = len(datum)
                        while i < 15:
                            datum.append(0)
                            i=i+1

                    self.presets.append(datum)
                
                f.close()

                
            def save_presets():
                f =  open(sys.path[0]+'/formula_presets2.csv','wb')
                data = csv.writer(f, delimiter=',',quotechar='"')
                self.presets=sorted(self.presets,key=lambda x: str.lower(x[0]))
                for preset in self.presets:
                    data.writerow(preset)

                f.close()
                
            def update_list():
                 self.items=[]
                 for preset in self.presets:
                     self.items.append(preset[0])
            
                 self.items = sorted(self.items,key=str.lower)
                 #populate list box with names
                 pres_list.delete(0, END)
                 for item in self.items:
                     pres_list.insert(END, item)

                 pres_list.select_set(0)

            def load_preset():
                if len(pres_list.curselection())>0:
                    
                    presets=self.presets

                    for preset in self.presets:
                        if preset[0] == pres_list.get(pres_list.curselection()):
                            name_fld.delete(0,END)
                            name_fld.insert(0,preset[0])
                            side1_x_fld.delete(0,END)
                            side1_x_fld.insert(0,preset[1])
                            side1_y_fld.delete(0,END)
                            side1_y_fld.insert(0,preset[2])
                            side1_p_fld.delete(0,END)
                            side1_p_fld.insert(0,preset[3])

                            side2_x_fld.delete(0,END)
                            side2_x_fld.insert(0,preset[4])
                            side2_y_fld.delete(0,END)
                            side2_y_fld.insert(0,preset[5])
                            side2_p_fld.delete(0,END)
                            side2_p_fld.insert(0,preset[6])

                            side3_x_fld.delete(0,END)
                            side3_x_fld.insert(0,preset[7])
                            side3_y_fld.delete(0,END)
                            side3_y_fld.insert(0,preset[8])
                            side3_p_fld.delete(0,END)
                            side3_p_fld.insert(0,preset[9])

                            side4_x_fld.delete(0,END)
                            side4_x_fld.insert(0,preset[10])
                            side4_y_fld.delete(0,END)
                            side4_y_fld.insert(0,preset[11])
                            side4_p_fld.delete(0,END)
                            side4_p_fld.insert(0,preset[12])
                            clos_p_fld.delete(0,END)
                            clos_p_fld.insert(0,preset[13])
                            offset_p_fld.delete(0,END)
                            offset_p_fld.insert(0,preset[14])

                            update(self)
    
            def find(f, seq): 
               """Return first item in sequence where f(item) == True."""
               for item in seq:
                   if f==item:
                       return True
                       
                    
            def save_preset():
                exists = find(name_fld.get(), self.items)
                if exists:
                    msg.showwarning("Name conflict","Preset name already in use.")

                else:
                    presets = self.presets
                    new = []
                    new.append(name_fld.get())           
                    new.append(side1_x_fld.get())
                    new.append(side1_y_fld.get())
                    new.append(side1_p_fld.get())

                    new.append(side2_x_fld.get())
                    new.append(side2_y_fld.get())
                    new.append(side2_p_fld.get())

                    new.append(side3_x_fld.get())
                    new.append(side3_y_fld.get())
                    new.append(side3_p_fld.get())

                    new.append(side4_x_fld.get())
                    new.append(side4_y_fld.get())
                    new.append(side4_p_fld.get())

                    new.append(clos_p_fld.get())
                    new.append(offset_p_fld.get())

                    #replace empty values with zeroes if any

                    for i in range(len(new)):
                        if new[i] == '':
                            new[i]=0

                    
                    presets.append(new)

                    save_presets()
                    update_list()
            
            def delete_preset():
                name = pres_list.get(pres_list.curselection())
                presets = self.presets
                index=0
                for preset in presets:
                    if preset[0]==name:
                        result = msg.askokcancel(title="Delete Preset", 
                                       message="Delete preset?")
                        if result:
                            del presets[index]

                    index = index+1

                save_presets()
                update_list()

            def copy_points():
                tmp = Tk()
                tmp.withdraw()
                points_data = []
                # check for offset value and trim list if needed
                for i in range(len(self.data)):
                    if i >= int(offset_p_fld.get()):
                        points_data.append(self.data[i])
                  
                tmp.clipboard_clear()
                tmp.clipboard_append(points_data)
                tmp.destroy()

            # Callbacks
            def fld(name):
                if name.get() == '':
                    name.insert(0,'0')
                    return '0'
                else:
                    return name.get()
                
            def update(event):
                
                side1_points = int(fld(side1_p_fld))
                side2_points = int(fld(side2_p_fld))
                side3_points = int(fld(side3_p_fld))
                side4_points = int(fld(side4_p_fld))
                closing_points = int(fld(clos_p_fld))
                offset_points = int(fld(offset_p_fld))

                self.offset_points = offset_points
                
                points = side1_points + side2_points + side3_points + side4_points + closing_points

                side1_xf = fld(side1_x_fld)
                side1_yf = fld(side1_y_fld)

                side2_xf = fld(side2_x_fld)
                side2_yf = fld(side2_y_fld)

                side3_xf = fld(side3_x_fld)
                side3_yf = fld(side3_y_fld)

                side4_xf = fld(side4_x_fld)
                side4_yf = fld(side4_y_fld)

                
                self.process(side1_xf, side1_yf,side1_points,
                 side2_xf, side2_yf,side2_points,
                 side3_xf, side3_yf,side3_points,
                 side4_xf, side4_yf,side4_points, points,canvas)

            def select_active(event):
                pres_list.select_clear(pres_list.curselection())
                pres_list.select_set(ACTIVE)
                load_preset()


            def scroll_to_letter(event):
                index=0

                for item in self.items:
                    
                    if(item[0:1].lower()== event.char.lower()):
                        pres_list.select_clear(pres_list.curselection())
                        pres_list.activate(index)
                        pres_list.select_set(index)
                        pres_list.see(index)
                        break

                    index=index+1

            #store presets in a list
            self.presets=[]
            open_presets()
            self.items=[]
            
            # Create GUI window ------------------------------
            
            win = Tk()
            win.title ("Formula Viewer v2.0")

            panes = PanedWindow(orient=HORIZONTAL,sashrelief="raised")
            panes.pack(fill=BOTH, expand=1)
            main_frame = Frame()

            
            ctrl_frame = Frame(main_frame, relief="ridge")
            name_lbl = Label(ctrl_frame,text="Name:")
            name_lbl.grid(row=0)

            name_fld = Entry(ctrl_frame,width=20)
            name_fld.insert(0,"Untitled")
            name_fld.grid(row=0, column=1,sticky=W)

            # --------------------------------------------------------------------
            side1 = Label(ctrl_frame,text="Side 1")
            side1.grid(row=1, columnspan=3)
            
            side1_x_lbl = Label(ctrl_frame,text="X:")
            side1_x_lbl.grid(row=2)

            side1_x_fld = Entry(ctrl_frame,width=50)
            side1_x_fld.bind("<Return>",update)
            side1_x_fld.insert(1,"(i*0.8)*4")
            side1_x_fld.grid(row=2, column=1)

            side1_y_lbl = Label(ctrl_frame,text="Y:")
            side1_y_lbl.grid(row=3)
            
            side1_y_fld = Entry(ctrl_frame,width=50)
            side1_y_fld.insert(0,"200+(sin(i*0.2)*20)")
            side1_y_fld.bind("<Return>",update)
            side1_y_fld.grid(row=3, column=1)

            side1_p_lbl = Label(ctrl_frame,text="Points:")
            side1_p_lbl.grid(row=4)
            
            side1_p_fld = Spinbox(ctrl_frame,width=10,from_=10, to=10000)
            side1_p_fld.delete(0,END)
            side1_p_fld.insert(0,50)
            side1_p_fld.bind("<Return>",update)
            side1_p_fld.grid(row=4,column=1,sticky=W)
            
            # --------------------------------------------------------------
            side2 = Label(ctrl_frame,text="Side 2")
            side2.grid(row=5, columnspan=3)
            
            side2_x_lbl = Label(ctrl_frame,text="X:")
            side2_x_lbl.grid(row=6)

            side2_x_fld = Entry(ctrl_frame,width=50)
            side2_x_fld.bind("<Return>",update)
            side2_x_fld.insert(1,"(i*0.8)*4")
            side2_x_fld.grid(row=6, column=1)

            side2_y_lbl = Label(ctrl_frame,text="Y:")
            side2_y_lbl.grid(row=7)
            
            side2_y_fld = Entry(ctrl_frame,width=50)
            side2_y_fld.insert(0,"200+(sin(i*0.2)*20)")
            side2_y_fld.bind("<Return>",update)
            side2_y_fld.grid(row=7, column=1)

            side2_p_lbl = Label(ctrl_frame,text="Points:")
            side2_p_lbl.grid(row=8)
            
            side2_p_fld = Spinbox(ctrl_frame,width=10,from_=10, to=10000)
            side2_p_fld.delete(0,END)
            side2_p_fld.insert(0,50)
            side2_p_fld.bind("<Return>",update)
            side2_p_fld.grid(row=8,column=1,sticky=W)

            # -------------------------------------------------------------------------
            side3 = Label(ctrl_frame,text="Side 3")
            side3.grid(row=9, columnspan=3)
            
            side3_x_lbl = Label(ctrl_frame,text="X:")
            side3_x_lbl.grid(row=10)

            side3_x_fld = Entry(ctrl_frame,width=50)
            side3_x_fld.bind("<Return>",update)
            side3_x_fld.insert(1,"(i*0.8)*4")
            side3_x_fld.grid(row=10, column=1)

            side3_y_lbl = Label(ctrl_frame,text="Y:")
            side3_y_lbl.grid(row=11)
            
            side3_y_fld = Entry(ctrl_frame,width=50)
            side3_y_fld.insert(0,"200+(sin(i*0.2)*20)")
            side3_y_fld.bind("<Return>",update)
            side3_y_fld.grid(row=11, column=1)

            side3_p_lbl = Label(ctrl_frame,text="Points:")
            side3_p_lbl.grid(row=12)
            
            side3_p_fld = Spinbox(ctrl_frame,width=10,from_=10, to=10000)
            side3_p_fld.delete(0,END)
            side3_p_fld.insert(0,50)
            side3_p_fld.bind("<Return>",update)
            side3_p_fld.grid(row=12,column=1,sticky=W)

            # -------------------------------------------------------------------------
            side4 = Label(ctrl_frame,text="Side 4")
            side4.grid(row=13, columnspan=3)
            
            side4_x_lbl = Label(ctrl_frame,text="X:")
            side4_x_lbl.grid(row=14)

            side4_x_fld = Entry(ctrl_frame,width=50)
            side4_x_fld.bind("<Return>",update)
            side4_x_fld.insert(1,"(i*0.8)*4")
            side4_x_fld.grid(row=14, column=1)

            side4_y_lbl = Label(ctrl_frame,text="Y:")
            side4_y_lbl.grid(row=15)
            
            side4_y_fld = Entry(ctrl_frame,width=50)
            side4_y_fld.insert(0,"200+(sin(i*0.2)*20)")
            side4_y_fld.bind("<Return>",update)
            side4_y_fld.grid(row=15, column=1)

            side4_p_lbl = Label(ctrl_frame,text="Points:")
            side4_p_lbl.grid(row=16)
            
            side4_p_fld = Spinbox(ctrl_frame,width=10,from_=10, to=10000)
            side4_p_fld.delete(0,END)
            side4_p_fld.insert(0,0)
            side4_p_fld.bind("<Return>",update)
            side4_p_fld.grid(row=16,column=1,sticky=W)

            clos_p_lbl = Label(ctrl_frame,text="Closing Points:")
            clos_p_lbl.grid(row=17)
            
            clos_p_fld = Spinbox(ctrl_frame,width=10,from_=0, to=10000)
            clos_p_fld.delete(0,END)
            clos_p_fld.insert(0,0)
            clos_p_fld.bind("<Return>",update)
            clos_p_fld.grid(row=17,column=1,sticky=W)

            offset_p_lbl = Label(ctrl_frame,text="Offset Start:")
            offset_p_lbl.grid(row=18)
            
            offset_p_fld = Spinbox(ctrl_frame,width=10,from_=0, to=10000)
            offset_p_fld.delete(0,END)
            offset_p_fld.insert(0,0)
            offset_p_fld.bind("<Return>",update)
            offset_p_fld.grid(row=18,column=1,sticky=W)

            copy_btn = Button(ctrl_frame, text="Copy Points",
                           command=copy_points, pady=4,padx=4)
            copy_btn.grid(row=19,padx=5, pady=5)
            
            ctrl_frame.pack(padx=10,pady=10)
            
            # -------------------------------------------------------------------------

            canv_frame = Frame( relief="ridge")
##            canvas = Canvas(main_frame,width=Width,height=Height)
            canvas = Canvas(canv_frame,bd=2, relief="sunken", width=Width,height=Height)
##            canvas.grid(row=19, columnspan=3,sticky=W+E+N+S)
            canvas.pack(fill=BOTH, expand=1)
            note_lbl = Label(canv_frame,text="Use this tool to create your curve's formulae (hit ENTER to refresh), then paste to the plugin dialog fields.")
            note_lbl.pack(expand=TRUE)

##            canv_frame.pack(side=RIGHT, fill=BOTH, expand=TRUE)
            
                    

            # Separator
            s = Frame(main_frame, relief=SUNKEN,height=1,bg="black")
            s.pack(fill=X, expand=True)

            # Presets area
            pres_frame = Frame(main_frame, relief=RAISED)
            
            # Label
            pres_lbl = Label(pres_frame,text="Presets:")
            pres_lbl.grid(row=0)

            #Dropdown frame
            list_frame = Frame(pres_frame)
            # Srollbar
            pres_scroll = Scrollbar(list_frame)
            pres_scroll.pack(side=RIGHT, fill=Y)
            #Preset List Box
            pres_list = Listbox(list_frame, selectmode=SINGLE)
            # List of preset names

            update_list()
            pres_list.pack()

##            main_frame.pack()
            panes.add(main_frame)
            panes.add(canv_frame)

            
            #bind scrollbar to the list box
            pres_scroll.config(command = pres_list.yview)
            pres_list.config(height=3,yscrollcommand=pres_scroll.set)


            pres_list.bind("<Return>",select_active)
            pres_list.bind("<KeyPress>",scroll_to_letter)
          
            list_frame.grid(row=1,column=0,padx=10, pady=10)

            #Load button
            l_btn = Button(pres_frame, text="Load",
                           command=load_preset, pady=4,padx=4)
            l_btn.grid(row=1,column=1,padx=5, pady=5)

            #Delete button
            l_btn = Button(pres_frame, text="Delete",
                           command=delete_preset, pady=4,padx=4)
            l_btn.grid(row=1,column=2,padx=5, pady=5)

            #Save button
            l_btn = Button(pres_frame, text="Save",
                           command=save_preset, pady=4,padx=4)
            l_btn.grid(row=1,column=3,padx=5, pady=5)


            pres_frame.pack(fill=X)

            # End of GUI window code ------------------------------


            side1_points = int(side1_p_fld.get())
            side1_xf = side1_x_fld.get()
            side1_yf = side1_y_fld.get()

            side2_points = int(side2_p_fld.get())
            side2_xf = side2_x_fld.get()
            side2_yf = side2_y_fld.get()

            side3_points = int(side3_p_fld.get())
            side3_xf = side3_x_fld.get()
            side3_yf = side3_y_fld.get()

            side4_points = int(side4_p_fld.get())
            side4_xf = side4_x_fld.get()
            side4_yf = side4_y_fld.get()

            closing_points = int(clos_p_fld.get())

            points = side1_points + side2_points + side3_points + side4_points + closing_points
            
            self.data = [[0.0,0.0] for i in range(side1_points)]
            self.points = points

            self.process(side1_xf, side1_yf,side1_points,
                         side2_xf, side2_yf,side2_points,
                         side3_xf, side3_yf,side3_points,
                         side4_xf, side4_yf,side4_points,points,canvas)

            win.mainloop()
      
    def process (self,side1_xf, side1_yf,side1_points,
                 side2_xf, side2_yf,side2_points,
                 side3_xf, side3_yf,side3_points,
                 side4_xf, side4_yf,side4_points, points,
                 canvas,Width=400, Height=400):
        self.data = [[0.0,0.0] for i in range(points)]
        self.points = points

        phi = 1.618   #1.61803399
        phi_inv = 0.618    #0.6180340
        
        dif_x = None
        dif_y = None
        dif_x2 = None
        dif_y2 = None
        dif_x3 = None
        dif_y3 = None
        dif_x4 = None
        dif_y4 = None
        
        for p in range(points):
            i = p

            try:
                if i<side1_points:                    
                    self.data[i][0]=eval(side1_xf)
                    self.data[i][1]=eval(side1_yf)

                elif i<side1_points+side2_points:

                    #calculate difference
                    if not dif_x:
                        dif_x = self.data[i-1][0] - eval(side2_xf)
                        dif_y = self.data[i-1][1] - eval(side2_yf)

                    self.data[i][0]=eval(side2_xf) + dif_x
                    self.data[i][1]=eval(side2_yf) + dif_y

                elif i < side1_points + side2_points + side3_points:
                    if not dif_x2:
                        dif_x2 = self.data[i-1][0] - eval(side3_xf)
                        dif_y2 = self.data[i-1][1] - eval(side3_yf)

                    self.data[i][0]=eval(side3_xf) + dif_x2
                    self.data[i][1]=eval(side3_yf) + dif_y2

                elif i < side1_points + side2_points + side3_points + side4_points:
                    if not dif_x3:
                        dif_x3 = self.data[i-1][0] - eval(side4_xf)
                        dif_y3 = self.data[i-1][1] - eval(side4_yf)

                    self.data[i][0]=eval(side4_xf) + dif_x3
                    self.data[i][1]=eval(side4_yf) + dif_y3

                    # closing points for seamless effect
                elif i < points:
                    if not dif_x4:
                        dif_x4 = self.data[i-1][0] - eval(side1_xf)
                        dif_y4 = self.data[i-1][1] - eval(side1_yf)

                    self.data[i][0]=eval(side1_xf) + dif_x4
                    self.data[i][1]=eval(side1_yf) + dif_y4
                
                    
            except:
                msg.showwarning("Error","Your formula has an error")
                break
            
        self.plotOne (canvas,Width,Height)



    def plotOne (self, canvas, Width, Height) :
            #items = canvas.find_all()
            canvas.delete(ALL)
            for i in range(len(self.data)) :
                x = self.data[i][0]
                y = self.data[i][1]
                if i > self.offset_points:
                    if i : canvas.create_line(self.data[i-1][0],yprev,x,y)
                    
                yprev = y
def run():
    inst = fview(512,512)

if __name__ == "__main__" : run()
